/*  libcalenabler - Library of calendaring and scheduling enabler
 *  calevent.h - a set of functions associated with event
 *
 *  Authors: YANG Lin <lin.yang@orange-ftgroup.com> 
 *
 *  This software and associated documentation files (the "Software") 
 *  are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *  All Rights Reserved. 
 *
 *  A copyright license is hereby granted for redistribution and use of 
 *  the Software in source and binary forms, with or without modification, 
 *  provided that the following conditions are met: 
 *  - Redistributions of source code must retain the above copyright notice, 
 *  this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 *   notice, this copyright license and the following disclaimer in the 
 *  documentation and/or other materials provided with the distribution. 
 *  - Neither the name of LiPS  nor the names of its Members may be used 
 *  to endorse or promote products derived from the Software without 
 *  specific prior written permission. 
 *
 *  A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *  to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *  distribute any implementation compliant with the any specification adopted 
 *  by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *  and non-discriminatory terms and conditions and under reciprocity, as 
 *  regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *  "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *  AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *  ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *  POSSIBILITY OF SUCH DAMAGE. 
 */


#ifndef __LIPS_CAL_EVENT_H
#define __LIPS_CAL_EVENT_H

#include <calbase.h>
#include <caltemplate.h>

#ifdef  HAS_VCALENDAR_SUPPORT
#include <mimedir.h>
#endif

#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */
/**
 * @brief Structure to hold event.
 *
 * The instance structure _cal_event_t must be defined in
 * implementation source code.
 */
  typedef struct _cal_event_t cal_event_t;



/** cal_event_t field ids.
* Each field has a unique id.
* The enabler predefined some fields.
* Application can also add new customized field to event using the function:
*   event_add_field ();
* To remove customized field,use this function:
*    event_remove_field ();
*/
#define    EVENT_TID_START            CAL_TEMPLATE_ID_START
/** cal_event_t field id - uid.
* Each event has a unique uid.
*/
#define    EVENT_UID                  EVENT_TID_START
/** cal_event_t field id - description.
* description field
*/
#define    EVENT_DESCRIPTION          (EVENT_UID+1)
/** cal_event_t field id - location.
* location field
*/
#define    EVENT_LOCATION            (EVENT_UID+2)
/** cal_event_t field id - notes.
* notes field
*/
#define    EVENT_NOTES                (EVENT_UID+3)
/** cal_event_t field id - start time.
* start time field
*/
#define    EVENT_START_DATETIME      (EVENT_UID+4)
/** cal_event_t field ids - end time.
* end time field
*/
#define    EVENT_END_DATETIME        (EVENT_UID+5)
/** cal_event_t field id - has alarm 
* has alarm field
*/

#define    EVENT_HAS_ALARM            (EVENT_UID+6)
/** cal_event_t field ids - alarm delay
* alarm delay field
*/
#define    EVENT_ALARM_DELAY          (EVENT_UID+7)
/** cal_event_t field ids - alarm type.
* alarm type field
*/
#define    EVENT_ALARM_TYPE          (EVENT_UID+8)
/** cal_event_t field ids - repeat type.
* repeat type field
*/

#define    EVENT_REPEAT_TYPE          (EVENT_UID+9)
/** cal_event_t field ids - repeat frequency.
* repeat frequency field
*/
#define    EVENT_REPEAT_FREQUENCY    (EVENT_UID+10)
/** cal_event_t field ids - repeat forever.
* repeat forever field
*/
#define    EVENT_REPEAT_FOREVER      (EVENT_UID+11)
/** cal_event_t field ids - repeat end date.
* repeat end date field
*/
#define    EVENT_REPEAT_END_DATE      (EVENT_UID+12)
/** cal_event_t field ids - repeat weekdays.
* repeat weekdays field
*/
#define    EVENT_REPEAT_WEEKDAYS      (EVENT_UID+13)
/** cal_event_t field ids - repeat monthdays.
* repeat monthdays field
*/
#define    EVENT_REPEAT_MONTHDAYS    (EVENT_UID+14)
/** cal_event_t field ids - repeat on nearest.
* if the repeat day doesn't exist, whether repeat on nearest day
*/
#define    EVENT_REPEAT_ON_NEAREST    (EVENT_UID+15)
/** cal_event_t field ids - has exception.
* repeat has exception field
*/
#define    EVENT_HAS_EXCEPTION        (EVENT_UID+16)
/** cal_event_t field ids - tid end.
* tid end
*/
#define    EVENT_TID_END             EVENT_HAS_EXCEPTION

/** cal_event_t alarm type.
* There're 2 alarm types:
*   ALARM_SILENT: no sound;
*    ALARM_LOUD: sound is produced when alarm;
*/
#define    ALARM_SILENT              0
/** cal_event_t alarm type.
* There're 2 alarm types:
*   ALARM_SILENT: no sound;
*    ALARM_LOUD: sound is produced when alarm;
*/
#define    ALARM_LOUD                1

/** cal_event_t repeat type.
* There're 5 repeat types:
*   REPEAT_NONE: just occur once, no repeat;
*    REPEAT_DAILY: repeat every day;
*    REPEAT_WEEKLY: repeat every week;
*    REPEAT_MONTHLY: repeat every month;
*    REPEAT_YEARLY: repeat every year;
*/
#define    REPEAT_NONE                0
/** cal_event_t repeat type.
*   REPEAT_DAILY: repeat every day;
*/
#define    REPEAT_DAILY               1
/** cal_event_t repeat type.
*    REPEAT_WEEKLY: repeat every week;
*/
#define    REPEAT_WEEKLY             2
/** cal_event_t repeat type.
*    REPEAT_MONTHLY: repeat every month;
*/
#define    REPEAT_MONTHLY             3
/** cal_event_t repeat type.
*    REPEAT_YEARLY: repeat every year;
*/
#define    REPEAT_YEARLY              4


/** @brief create a new event.
*
* Use this function to create a new event.
*
* @return   cal_event_t  if success, NULL if failure.
*
* @note     In this function,memory is allocated for event, remember to free 
*           the memory by event_free when no longer needed.
*/
  cal_event_t *event_new (void);
/** @brief free a event.
*
* Use this function to free a event.
*
* @param    cal_event_t the event to free     
*
*/
  void event_free (cal_event_t *event);
/** @brief set description for event.
*
* Use this function to set descripton for a event.
*
* @param    event          pointer to the event to set description
* @param    description   the description string to set     
*/
  void event_set_description (cal_event_t *event, const cal_char_t *description);
/** @brief set location for event.
*
* Use this function to set location for a event.
*
* @param    event          pointer to the event to set location
* @param    location       the location string to set     
*/
  void event_set_location (cal_event_t *event, const cal_char_t *location);
/** @brief set notes for event.
*
* Use this function to set notes for a event.
*
* @param    event          pointer to the event to set notes
* @param    notes         the notes string to set     
*/
  void event_set_notes (cal_event_t *event, const cal_char_t *notes);
/** @brief set start date and time for event.
*
* Use this function to set start date and time for a event.
*
* @param    event          pointer to the event to set start date and time
* @param    start         the start date and time to set     
*/
  void event_set_start (cal_event_t *event, const cal_time_t start);
/** @brief set end date and time for event.
*
* Use this function to set end date and time for a event.
*
* @param    event          pointer to the event to set end date and time
* @param    end             the end date and time to set     
*/
  void event_set_end (cal_event_t *event, const cal_time_t end);
/** @brief set whether the event has alarm.
*
* Use this function to set whether the event has alarm.
*
* @param    event          pointer to the event 
* @param    setting         set whether the event has alarm     
*/
  void event_set_has_alarm (cal_event_t *event, cal_boolean_t setting);
/** @brief set alarm_type for event.
*
* Use this function to set alarm_type for a event.
*
* @param    event          pointer to the event to set alarm delay and alarm_type 
* @param    alarm_type    alarm type, it can only be the value of pre-defined constants
*/
  void event_set_alarm_type (cal_event_t *event, cal_uint32_t alarm_type);
/** @brief set alarm delay for event.
*
* Use this function to set alarm delay for a event.
*
* @param    event          pointer to the event to set alarm delay and alarm_type
* @param    delay           alarm delay     
*/
  void event_set_alarm_delay (cal_event_t *event, cal_uint32_t delay);
/** @brief clear alarm for event.
*
* Use this function to clear alarm for a event.
*
* @param    event          pointer to the event 
*/
  void event_clear_alarm (cal_event_t *event);
/** @brief set repeat type for event.
*
* Use this function to set repeat type for a event.
*
* @param    event          pointer to the event to set repeat type
* @param    repeat_type    repeat type, it can only be the value of pre-defined constants
*/
  void event_set_repeat_type (cal_event_t *event, cal_uint32_t repeat_type);
/** @brief set repeat frequency for event.
*
* If repeat type is REPEAT_DAILY or REPEAT_YEARLY,use this function to set repeat frequency for a event.
*
* @param    event          pointer to the event to set repeat frequency
* @param    frequency      repeat frequency
*/
  void event_set_repeat_frequency (cal_event_t *event, cal_uint32_t frequency);
/** @brief set repeat end for event.
*
* If repeat type isn't  REPEAT_NONE ,use this function to set repeat end date and time for a event.
*
* @param    event          pointer to the event to set repeat end
* @param    end            repeat end date and time
*/
  void event_set_repeat_end (cal_event_t *event, const cal_time_t end);
/** @brief set whether the event repeat forever.
*
* If repeat type isn't  REPEAT_NONE ,use this function to set whether the event repeat forever.
*
* @param    event          pointer to the event 
* @param    setting        set whether the event repeat forever
*/
  void event_set_repeat_forever (cal_event_t *event, cal_boolean_t setting);
/** @brief set the event repeat weekdays.
*
* If repeat type is REPEAT_WEEKLY ,use this function to set the event repeat weekdays.
*
* @param    event          pointer to the event 
* @param    weekdays      set the event repeat weekdays,it's a bit mask with the lowst bit representing Monday and so on
*/
  void event_set_repeat_weekdays (cal_event_t *event, cal_uint8_t weekdays);

/** @brief set the event repeat monthdays.
*
* If repeat type is REPEAT_MONTHLY ,use this function to set the event repeat monthdays.
*
* @param    event          pointer to the event 
* @param    monthdays      set the event repeat monthdays,it's a bit mask with the lowst bit representing the first day of the month,and so on.
*/
  void event_set_repeat_monthdays (cal_event_t *event, cal_uint32_t monthdays);

/** @brief set whether the repeating event has exceptions.
*
*The function set whether to show a repeating event on the nearest previous date if the day it would repeat doesn't 
*exist.For example, a repeating event happen on 31 day of every month.
*
* @param    event          pointer to the event 
* @param    setting        set whether the repeating event has exceptions.
*/
  void event_set_repeat_on_nearest (cal_event_t *event, cal_boolean_t setting);

/** @brief set whether the repeating event has exceptions.
*
* If repeat type isn't REPEAT_NONE ,use this function to set whether the repeating event has exceptions.
*
* @param    event          pointer to the event 
* @param    setting        set whether the repeating event has exceptions.
*/
  void event_set_has_exception (cal_event_t *event, cal_boolean_t setting);
/** @brief add exceptions to repeating event.
*
* If repeat type isn't REPEAT_NONE ,use this function to add exception to repeating event.One repeat event could has more than one exceptions.
*
* @param    event          pointer to the event 
* @param    exception      the exception to add to event
*/
  void event_add_exception (cal_event_t *event, cal_time_t exception);
/** @brief add event to a category.
*
* Use this function to add event to a category.An event can be added to more than one category.
*
* @param    event          pointer to the event 
* @param    cid            the category id 
*/
  void event_add_category (cal_event_t *event, const cal_char_t *cid);
/** @brief get event uid.
*
* Get event uid.The returned string is newly-allocated.It should be freed when no longer needed.
*
* @param    event          pointer to the event 
* @return   the event uid if success
* @return   NULL if failure.
*/
  cal_char_t *event_get_uid (const cal_event_t *event);
/** @brief get event description.
*
* Get event description.The returned string is newly-allocated.It should be freed when no longer needed.
*
* @param    event          pointer to the event 
* @return   the event description if success
* @return   NULL if failure.
*/
  cal_char_t *event_get_description (const cal_event_t *event);
/** @brief get event location.
*
* Get event location.The returned string is newly-allocated.It should be freed when no longer needed.
*
* @param    event          pointer to the event 
* @return   the event location if success
* @return   NULL if failure.
*/
  cal_char_t *event_get_location (const cal_event_t *event);
/** @brief get event notes.
*
* Get event notes.The returned string is newly-allocated.It should be freed when no longer needed.
*
* @param    event          pointer to the event 
* @return   the event notes if success
* @return   NULL if failure.
*/
  cal_char_t *event_get_notes (const cal_event_t *event);
/** @brief get event start time.
*
* Get event start date and time.
*
* @param    event          pointer to the event 
* @return   the event start time if success
* @return   0 if failure.
*/
  cal_time_t event_get_start (const cal_event_t *event);
/** @brief get event end time.
*
* Get event end time.
*
* @param    event          pointer to the event 
* @return   the event end time if success
* @return   0 if failure.
*/
  cal_time_t event_get_end (const cal_event_t *event);
/** @brief get whether event has alarm.
*
* Get whether event has alarm.
*
* @param    event          pointer to the event 
* @return   TRUE if has alarm
* @return   FALSE if event doesn't have alarm
*/
  cal_boolean_t event_get_has_alarm (const cal_event_t *event);
/** @brief get event alarm delay.
*
* Get event alarm delay.
*
* @param    event          pointer to the event 
* @return   alarm delay
*/
  cal_uint32_t event_get_alarm_delay (const cal_event_t *event);
/** @brief get event alarm type.
*
* Get event alarm type.
*
* @param    event          pointer to the event 
* @return   alarm type
*/
  cal_uint32_t event_get_alarm_type (const cal_event_t *event);
/** @brief get event repeat type.
*
* Get event repeat type.
*
* @param    event          pointer to the event 
* @return   repeat type
*/
  cal_uint32_t event_get_repeat_type (const cal_event_t *event);
/** @brief get event repeat frequency.
*
* Get event repeat frequency.
*
* @param    event          pointer to the event 
* @return   repeat frequency
*/
  cal_uint32_t event_get_repeat_frequency (const cal_event_t *event);
/** @brief get event repeat end.
*
* Get event repeat end.
*
* @param    event          pointer to the event 
* @return   repeat end
*/
  cal_time_t event_get_repeat_end (const cal_event_t *event);
/** @brief get whether event repeat forever.
*
* If event repeat type isn't REPEAT_NONE,get whether event repeat forever.
*
* @param    event          pointer to the event 
* @return   TRUE if repeat forever
* @return   FALSE if not
*/
  cal_boolean_t event_get_repeat_forever (const cal_event_t *event);
/** @brief get event repeat weekdays.
*
* If event repeat type is REPEAT_WEEKLY, get event repeat weekdays.
*
* @param    event          pointer to the event 
* @return   repeat weekdays
*/
  cal_uint8_t event_get_repeat_weekdays (const cal_event_t *event);
/** @brief get event repeat weekdays.
*
* If event repeat type is REPEAT_MONTHLY, get event repeat monthdays.
*
* @param    event          pointer to the event 
* @return   repeat monthdays
*/
  cal_uint32_t event_get_repeat_monthdays (const cal_event_t *event);
/** @brief get whether event repeat on nearest day
*
*
* @param    event          pointer to the event 
* @return   TRUE           if event repeat on nearest day
* @return   FALSE         if not  
*/
  cal_boolean_t event_get_repeat_on_nearest (const cal_event_t *event);
/** @brief get whether event has exceptions
*
*
* @param    event          pointer to the event 
* @return   TRUE           if event has exceptions
* @return   FALSE         if not  
*/
  cal_boolean_t event_get_has_exception (const cal_event_t *event);
/** @brief get categories the event belong to.
*
* get categories the event belong to.
*
* @param    event          pointer to the event 
* @return   the categories ids that the event belong to 
*/
  cal_char_t **event_get_category (const cal_event_t *event);
/** @brief get next occurence for repeat event.
*
* Get latest next occurence for repeat event on or after "from".If the event only 
* occurs once (no repeat) will return the date of the start of the event if the 
*  start of the event is on or after from. If ok is non-NULL, *ok is set to TRUE if 
*  the event occurs on or after from and FALSE if the event does not occur on or after from.
*
* @param    event          pointer to the event 
* @return   the the occurence time on or after "from"
*/
  cal_time_t event_get_next_occurence (const cal_event_t *event, cal_time_t after, 
                                       cal_boolean_t *ok);
/** @brief set customized field value for event.
*
* Use this function to set customized field value for event.
*
* @param    event          pointer to the event 
* @param    tid            the field id 
* @param    value          the field value 
* @return   cal error number 
*/
  void event_set_field (cal_event_t * event, cal_uint32_t tid, cal_pointer_t value);
/** @brief get customized field value for event.
*
* Use this function to get customized field value for event.Notes that the returned string is newly-allocated.It should be freed when no longer needed.
*
* @param    event          pointer to the event 
* @param    tid            the field id 
* @return   the pointer to the field value 
*/
  cal_pointer_t event_get_field (cal_event_t *event, cal_uint32_t tid);
#ifdef  HAS_VCALENDAR_SUPPORT
/** @brief convert cal_event_t structure to MIMEDirVEvent structure.
*
* Use this function to convert cal_event_t structure to MIMEDirVEvent structure which is defined by the vCalendar 1.0 specification.
*
* @param    event          pointer to the event 
* @param    vcal          the MIMEDirVcal_event_t structure
* @return   cal error number 
*/
  cal_err_t event_export_vcal (const cal_event_t *event, MIMEDirVEvent *vcal);
/** @brief convert MIMEDirVcal_event_t structure to cal_event_t structure.
*
* Use this function to convert MIMEDirVcal_event_t structure to cal_event_t structure 
*
* @param    event          pointer to the event 
* @param    vcal          the MIMEDirVcal_event_t structure
* @return   cal error number 
*/
  cal_err_t event_import_vcal (const MIMEDirVcal_event_t *vcal, cal_event_t *event);
#endif

/**
 * @brief Structure to hold event set.
 *
 * The instance structure _cal_event_iter_t must be defined in
 * implementation source code.
 */
  typedef struct _cal_event_iter_t cal_event_iter_t;

/** @brief create a new cal_event_iter_t.
*
* Use this function to create a new cal_event_iter_t.
*
* @return   cal_event_iter_t 
*/
  cal_event_iter_t *event_iterator_new (void);
/** @brief free a cal_event_iter_t.
*
* Use this function to free a cal_event_iter_t.In this function, not only the iterator, but also the real element data in the iterator are freed.
*
* @param    iter          pointer to the cal_event_iter_t 
*/
  void event_iterator_free (cal_event_iter_t *iter);
/** @brief get the number of cal_event_t in cal_event_iter_t.
*
* Use this function to get the number of cal_event_t in cal_event_iter_t.
*
* @param    iter          pointer to the cal_event_iter_t 
* @return   the number of cal_event_t in cal_event_iter_t. 
*/
  cal_int_t event_iterator_size (cal_event_iter_t *iter);
/** @brief return whether the pointer is set to the first event in cal_event_iter_t.
*
* This function return whether the pointer is set to the first event in cal_event_iter_t.
*
* @param    iter          pointer to the cal_event_iter_t 
* @return   TRUE if the  pointer is set to the first event in cal_event_iter_t, FALSE otherwise
*/
  cal_boolean_t event_iterator_at_first (const cal_event_iter_t *iter);
/** @brief return whether the pointer is set to the last event in cal_event_iter_t.
*
* This function return whether the pointer is set to the last event in cal_event_iter_t.
*
* @param    iter          pointer to the cal_event_iter_t 
* @return   TRUE if the  pointer is set to the last event in cal_event_iter_t, FALSE otherwise
*/
  cal_boolean_t event_iterator_at_last (const cal_event_iter_t *iter);
/** @brief move the pointer to the first event in cal_event_iter_t.
*
* This function move the pointer to the first event in the iterator. 
*
* @param    iter          pointer to the cal_event_iter_t 
*/
  void event_iterator_to_first (cal_event_iter_t *iter);
/** @brief move the pointer to the last event in cal_event_iter_t.
*
* This function move the pointer to the last event in the iterator. 
*
* @param    iter          pointer to the cal_event_iter_t 
*/
  void event_iterator_to_last (cal_event_iter_t *iter);
/** @brief get the element in cal_event_iter_t.
*
* This function returns a pointer to the element in iterator. Don't free the element directly.
Use event_iterator_free() to remove the whole iterator.
*
* @param    iter          pointer to the cal_event_iter_t 
* @return   a pointer to the cal_event_t in cal_event_iter_t. 
*/
  cal_event_t *event_iterator_current (const cal_event_iter_t *iter);
/** @brief move the pointer to the next event in cal_event_iter_t.
*
* This function move the pointer to the next event in the iterator. 
*
* @param    iter          pointer to the cal_event_iter_t 
*/
  void event_iterator_next (cal_event_iter_t *iter);

  void print_event (const cal_event_t *event);

#ifdef __cplusplus
}
#endif                          /* __cplusplus */

#endif                          /* __LIPS_CAL_EVENT_H */

/*
vi:ts=2:nowrap:ai:expandtab
*/

