/*  libcalenabler - Library of calendaring and scheduling enabler
 *
 *  Authors: YANG Lin <lin.yang@orange-ftgroup.com> 
 *  
 *  This software and associated documentation files (the "Software") 
 *  are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *  All Rights Reserved. 
 *
 *  A copyright license is hereby granted for redistribution and use of 
 *  the Software in source and binary forms, with or without modification, 
 *  provided that the following conditions are met: 
 *  - Redistributions of source code must retain the above copyright notice, 
 *  this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 *   notice, this copyright license and the following disclaimer in the 
 *  documentation and/or other materials provided with the distribution. 
 *  - Neither the name of LiPS  nor the names of its Members may be used 
 *  to endorse or promote products derived from the Software without 
 *  specific prior written permission. 
 *  A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *  to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *  distribute any implementation compliant with the any specification adopted 
 *  by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *  and non-discriminatory terms and conditions and under reciprocity, as 
 *  regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *  "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *  AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *  ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *  POSSIBILITY OF SUCH DAMAGE. 
 */


#ifndef  __EVENT_PRIVATE_H
#define __EVENT_PRIVATE_H

#include "caltype.h"
#include "calevent.h"
#include "caleventbook.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 *"repeat_on_nearest" set whether to show a repeating event on the nearest previous 
 *date if the day it would repeat doesn't exist.For example, a repeating event 
 *happen on 31 day of every month will be set to happen on the 28th or 29th day 
 *in February if "repeat_on_nearest" is TRUE.
 */
struct _cal_event_t
{
  cal_char_t *uid;
  cal_char_t *description;
  cal_char_t *location;
  cal_char_t *notes;
  cal_time_t start;
  cal_time_t end;

  cal_boolean_t has_alarm;
  cal_uint32_t alarm_delay;
  cal_uint32_t alarm_type;

  cal_uint32_t repeat_type;
  cal_uint32_t repeat_frequency;
  cal_boolean_t repeat_forever;
  cal_time_t repeat_end;
  cal_uint8_t repeat_weekdays;
  cal_uint32_t repeat_monthdays;
  cal_boolean_t repeat_on_nearest;

  cal_boolean_t has_exception;
  GPtrArray *exceptions;
  GPtrArray *categories;
  GHashTable *table;
};

void _event_set_uid (cal_event_t *event, const cal_char_t *uid);
cal_err_t _eventbook_add_exception (cal_eventbook_t *book, const cal_char_t *uid,
                                   cal_time_t exception);
cal_boolean_t _event_check_for_peroid (const cal_event_t *event, cal_time_t from,
                                      cal_time_t to);
void _event_iterator_insert (cal_event_iter_t *iter, cal_event_t *event);
cal_boolean_t _schedule_add_alarm (cal_eventbook_t *book, const cal_char_t *uid,
                                  cal_time_t time,
                                  const cal_char_t *description);
cal_boolean_t _schedule_cancel_alarm (cal_eventbook_t *book, const cal_char_t *uid);

void _print_event (const cal_event_t *event);
void _print_uid (const unsigned char *uid);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __EVENT_PRIVATE_H */
/*
vi:ts=2:nowrap:ai:expandtab
*/

