/*  libcalenabler - Library of calendaring and scheduling enabler
 *
 *  Authors: YANG Lin <lin.yang@orange-ftgroup.com> 
 *  
 *  This software and associated documentation files (the "Software") 
 *  are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *  All Rights Reserved. 
 *
 *  A copyright license is hereby granted for redistribution and use of 
 *  the Software in source and binary forms, with or without modification, 
 *  provided that the following conditions are met: 
 *  - Redistributions of source code must retain the above copyright notice, 
 *  this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 *   notice, this copyright license and the following disclaimer in the 
 *  documentation and/or other materials provided with the distribution. 
 *  - Neither the name of LiPS  nor the names of its Members may be used 
 *  to endorse or promote products derived from the Software without 
 *  specific prior written permission. 
 *
 *  A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *  to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *  distribute any implementation compliant with the any specification adopted 
 *  by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *  and non-discriminatory terms and conditions and under reciprocity, as 
 *  regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *  "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *  AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *  ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *  POSSIBILITY OF SUCH DAMAGE. 
 */

#include <glib.h>

#include "event-private.h"

#define INVALID_UID ""

cal_event_t *
event_new (void)
{
  cal_event_t *ev = NULL;
  ev = g_new0 (cal_event_t, 1);
  ev->exceptions = g_ptr_array_new ();
  ev->categories = g_ptr_array_new ();
  ev->table = g_hash_table_new_full (g_int_hash, g_int_equal, g_free, g_free);
  return ev;
}

void
event_free (cal_event_t *ev)
{
  g_free (ev->uid);
  g_free (ev->description);
  g_free (ev->location);
  g_free (ev->notes);
  g_ptr_array_free (ev->exceptions, TRUE);
  g_ptr_array_free (ev->categories, TRUE);
  g_hash_table_destroy (ev->table);
  g_free (ev);

  return;
}

void
_event_set_uid (cal_event_t *event, const cal_char_t *uid)
{
  event->uid = g_memdup (uid, 16);
}

void
event_set_description (cal_event_t *event, const cal_char_t *description)
{
  event->description = g_strdup (description);
}

void
event_set_location (cal_event_t *event, const cal_char_t *location)
{
  event->location = g_strdup (location);
}

void
event_set_notes (cal_event_t *event, const cal_char_t *notes)
{
  event->notes = g_strdup (notes);
}

void
event_set_start (cal_event_t *event, const cal_time_t start)
{
  event->start = start;
}

void
event_set_end (cal_event_t *event, const cal_time_t end)
{
  event->end = end;
}

void
event_set_has_alarm (cal_event_t *event, cal_boolean_t setting)
{
  event->has_alarm = setting;
}

void
event_set_alarm_type (cal_event_t *event, cal_uint32_t alarm_type)
{
  event->alarm_type = alarm_type;
}

void
event_set_alarm_delay (cal_event_t *event, cal_uint32_t delay)
{
  event->alarm_delay = delay;
}

void
event_clear_alarm (cal_event_t *event)
{
  event->has_alarm = FALSE;
}

void
event_set_repeat_type (cal_event_t *event, cal_uint32_t repeat_type)
{
  event->repeat_type = repeat_type;
}

void
event_set_repeat_frequency (cal_event_t *event, cal_uint32_t frequency)
{
  event->repeat_frequency = frequency;
}

void
event_set_repeat_forever (cal_event_t *event, cal_boolean_t setting)
{
  event->repeat_forever = setting;
}

void
event_set_repeat_end (cal_event_t *event, const cal_time_t end)
{
  event->repeat_end = end;
}

void
event_set_repeat_weekdays (cal_event_t *event, cal_uint8_t weekdays)
{
  event->repeat_weekdays = weekdays;
}

void
event_set_repeat_monthdays (cal_event_t *event, cal_uint32_t monthdays)
{
  event->repeat_monthdays = monthdays;
}

void
event_set_repeat_on_nearest (cal_event_t *event, cal_boolean_t setting)
{
  event->repeat_on_nearest = setting;
}

void
event_set_has_exception (cal_event_t *event, cal_boolean_t setting)
{
  event->has_exception = setting;
}

void
event_add_exception (cal_event_t *event, cal_time_t exception)
{
  cal_time_t *temp = g_malloc (sizeof (cal_time_t));

  *temp = exception;

  g_ptr_array_add (event->exceptions, (gpointer) temp);

  /*g_print (" %s:exception = %d\n", __FUNCTION__, *temp);*/
}

cal_char_t *
event_get_uid (const cal_event_t *event)
{
  cal_char_t *uid;
  uid = g_memdup (event->uid, 16);
  return uid;
}

cal_char_t *
event_get_description (const cal_event_t *event)
{
  return g_strdup (event->description);
}

cal_char_t *
event_get_location (const cal_event_t *event)
{
  return g_strdup (event->location);
}

cal_char_t *
event_get_notes (const cal_event_t *event)
{
  return g_strdup (event->notes);
}

cal_time_t
event_get_start (const cal_event_t *event)
{
  return event->start;
}

cal_time_t
event_get_end (const cal_event_t *event)
{
  return event->end;
}

cal_boolean_t
event_get_has_alarm (const cal_event_t *event)
{
  return event->has_alarm;
}

cal_uint32_t
event_get_alarm_delay (const cal_event_t *event)
{
  return event->alarm_delay;
}

cal_uint32_t
event_get_alarm_type (const cal_event_t *event)
{
  return event->alarm_type;
}

cal_uint32_t
event_get_repeat_type (const cal_event_t *event)
{
  return event->repeat_type;
}

cal_uint32_t
event_get_repeat_frequency (const cal_event_t *event)
{
  return event->repeat_frequency;
}

cal_time_t
event_get_repeat_end (const cal_event_t *event)
{
  return event->repeat_end;
}

cal_boolean_t
event_get_repeat_forever (const cal_event_t *event)
{
  return event->repeat_forever;
}

cal_uint8_t
event_get_repeat_weekdays (const cal_event_t *event)
{
  return event->repeat_weekdays;
}

cal_uint32_t
event_get_repeat_monthdays (const cal_event_t *event)
{
  return event->repeat_monthdays;
}

cal_boolean_t
event_get_repeat_on_nearest (const cal_event_t *event)
{
  return event->repeat_on_nearest;
}

cal_boolean_t
event_get_has_exception (const cal_event_t *event)
{
  return event->has_exception;
}

cal_time_t
event_get_next_occurence (const cal_event_t *event, cal_time_t after,
                          cal_boolean_t *ok)
{
  struct tm tm_after;
  struct tm tm_start;
  cal_time_t start = event->start;
  cal_time_t clone = start;

  localtime_r (&after, &tm_after);
  localtime_r (&start, &tm_start);

#ifdef DEBUG
  g_print ("%s, start =%d, after =%d\n", __FUNCTION__, start, after);
#endif
  if ((start - event->alarm_delay) > after)
  {
    *ok = TRUE;
    return clone;
  }
  if (!event->repeat_forever && event->repeat_end < after)
  {
    *ok = FALSE;
    return 0;
  }
#ifdef DEBUG
  g_print ("event->repeat_type =%d\n", (gint) event->repeat_type);
#endif
  switch (event->repeat_type)
  {
  case REPEAT_NONE:
    *ok = FALSE;
    break;
  case REPEAT_DAILY:
    {
      cal_int_t frq = event->repeat_frequency;
      cal_int_t between = tm_after.tm_mday - tm_start.tm_mday;
      gint i;

      tm_after.tm_mday += frq - between % frq;
      clone = mktime (&tm_after);
      g_print ("clone =%d\n", (int) clone);

      for (i = 0; i < event->exceptions->len; i++)
      {
        gpointer val = g_ptr_array_index (event->exceptions, i);
        if (clone == *(cal_time_t *) val)
        {
          tm_after.tm_mday += frq;
          clone = mktime (&tm_after);
        }
      }
      if ((clone <= event->repeat_end) || (event->repeat_forever))
        *ok = TRUE;
      else
        *ok = FALSE;
      break;
    }
  case REPEAT_WEEKLY:
    {
      cal_int_t weekdays = event->repeat_weekdays;
      cal_int_t wday = tm_after.tm_wday;
      gint i;

      for (i = 0; i < 7; i++)
      {
        wday += i;
        if (weekdays & (0x1 << ((wday - 1) % 7)))
          break;
      }
      if (i < 7)
      {
        tm_after.tm_mday += i;
        clone = mktime (&tm_after);
      }
      for (i = 0; i < event->exceptions->len; i++)
      {
        gpointer val = g_ptr_array_index (event->exceptions, i);
        if (clone == *(cal_time_t *) val)
        {
          clone = event_get_next_occurence (event, clone, ok);
        }
      }
      if ((clone > 0) &&
          ((clone <= event->repeat_end) || (event->repeat_forever)))
        *ok = TRUE;
      else
        *ok = FALSE;
      break;
    }
  case REPEAT_MONTHLY:
    {
      /*monthdays not supported yet */
      cal_time_t temp = start;
      struct tm tm_temp;
      gint i;

      localtime_r (&temp, &tm_temp);
      tm_temp.tm_mon = tm_after.tm_mon;
      tm_temp.tm_year = tm_after.tm_year;
      temp = mktime (&tm_temp);

      if (temp >= after)
        clone = temp;
      else
      {
        tm_temp.tm_mon++;
        clone = mktime (&tm_temp);
      }
      for (i = 0; i < event->exceptions->len; i++)
      {
        gpointer val = g_ptr_array_index (event->exceptions, i);
        if (clone == *(cal_time_t *) val)
        {
          tm_temp.tm_mon++;
          clone = mktime (&tm_temp);
        }
      }
      if ((clone <= event->repeat_end) || (event->repeat_forever))
        *ok = TRUE;
      else
        *ok = FALSE;
      break;
    }
  case REPEAT_YEARLY:
    {
      gint frq = event->repeat_frequency;
      cal_int_t between = tm_after.tm_year - tm_start.tm_year;
      cal_time_t temp = start;
      struct tm tm_temp;
      gint i;

      if (between % frq)
        break;

      localtime_r (&temp, &tm_temp);
      tm_temp.tm_year = tm_after.tm_year;
      temp = mktime (&tm_temp);

      if (temp >= after)
        clone = temp;
      else
      {
        tm_temp.tm_year = tm_after.tm_year + frq - between % frq;
        clone = mktime (&tm_temp);
      }
      for (i = 0; i < event->exceptions->len; i++)
      {
        gpointer val = g_ptr_array_index (event->exceptions, i);
        if (clone == *(cal_time_t *) val)
        {
          tm_temp.tm_year += frq;
          clone = mktime (&tm_temp);
        }
      }
      if ((clone <= event->repeat_end) || (event->repeat_forever))
        *ok = TRUE;
      else
        *ok = FALSE;
      break;
    }
  }                             /*switch (event->repeat_type)*/

  if (*ok)
    return clone;
  else
    return 0;
}



cal_pointer_t
event_get_field (cal_event_t *event, cal_uint32_t tid)
{

}
void
event_set_field (cal_event_t *event, cal_uint32_t tid, cal_pointer_t value)
{

}



/* "From " and "To " should be on the same day*/
cal_boolean_t
_event_check_for_peroid (const cal_event_t *event, cal_time_t from,
                         cal_time_t to)
{
  cal_boolean_t match = FALSE;
  struct tm tm_from;
  struct tm tm_start;
  cal_time_t clone_start, event_start = event->start;


  localtime_r (&from, &tm_from);
  /*g_print ("%s:tm_from.tm_mday=%d\n", __FUNCTION__, tm_from.tm_mday); */
  localtime_r (&event_start, &tm_start);

  switch (event->repeat_type)
  {
  case REPEAT_NONE:
    match = TRUE;
    break;
  case REPEAT_DAILY:
    {
      cal_int_t frq = event->repeat_frequency;

      if ((tm_start.tm_yday - tm_from.tm_yday) % frq == 0)
      {
        tm_start.tm_year = tm_from.tm_year;
        tm_start.tm_mon = tm_from.tm_mon;
        tm_start.tm_mday = tm_from.tm_mday;
        clone_start = mktime (&tm_start);
      }
    }
    break;
  case REPEAT_WEEKLY:
    {
      cal_int_t weekdays = event->repeat_weekdays;

      if (weekdays & (0x1 << (tm_from.tm_wday - 1)))
      {
        tm_start.tm_year = tm_from.tm_year;
        tm_start.tm_mon = tm_from.tm_mon;
        tm_start.tm_mday = tm_from.tm_mday;
        clone_start = mktime (&tm_start);
      }
      break;
    }
  case REPEAT_MONTHLY:
    {
      gint i;
      cal_int_t monthdays = event->repeat_monthdays;

      /*g_print ("%s:monthdays=%d, tm_from.tm_mday=%d\n", __FUNCTION__, monthdays,tm_from.tm_mday);   */

      if (monthdays & (0x1 << (tm_from.tm_mday - 1)))
      {
        tm_start.tm_year = tm_from.tm_year;
        tm_start.tm_mon = tm_from.tm_mon;
        tm_start.tm_mday = tm_from.tm_mday;
        clone_start = mktime (&tm_start);
      }
      break;
    }
  case REPEAT_YEARLY:
    {
      gint frq = event->repeat_frequency;


      if ((tm_start.tm_mon == tm_from.tm_mon)
          && (tm_start.tm_mday == tm_from.tm_mday)
          && ((tm_start.tm_year - tm_from.tm_year) % frq == 0))
      {
        tm_start.tm_year = tm_from.tm_year;
        clone_start = mktime (&tm_start);
      }
      break;
    }
  }                             /*switch (event->repeat_type)*/

  if (clone_start <= to && (clone_start + event->end - event->start) > from)
  {

    gint i;
    match = TRUE;
    for (i = 0; i < event->exceptions->len; i++)
    {
      cal_time_t *tempexcp;
      struct tm tm;

      tempexcp = g_ptr_array_index (event->exceptions, i);
      localtime_r (tempexcp, &tm);

      if ((tm.tm_year == tm_start.tm_year) &&
          (tm.tm_mon == tm_start.tm_mon) && (tm.tm_mday == tm_start.tm_mday))
        match = FALSE;

    }
  }
  return match;
}

char *
uid2str (unsigned char *uid)
{
  gchar *str;

  str =
    g_strdup_printf
    ("[%02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x]",
     uid[0], uid[1], uid[2], uid[3], uid[4], uid[5], uid[6], uid[7], uid[8],
     uid[9], uid[10], uid[11], uid[12], uid[13], uid[14], uid[15]);

  return str;
}

void
_print_uid (const unsigned char *uid)
{
  gchar *tempstr = NULL;

  /*g_print ("0x%08d\n", (guint32) uid);*/
  tempstr =
    g_strdup_printf
    ("[%02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x]",
     uid[0], uid[1], uid[2], uid[3], uid[4], uid[5], uid[6], uid[7], uid[8],
     uid[9], uid[10], uid[11], uid[12], uid[13], uid[14], uid[15]);

  g_print ("%s:uid:%-20s\n", __FUNCTION__, tempstr);

  g_free (tempstr);

}

void
_print_event (const cal_event_t * event)
{
  gchar *uid = NULL;
  gchar *uid_str = NULL;
  gchar *tmp_str = NULL;
  cal_time_t tmp_time;

  g_print ("-cal_event_t[0x%08x]:\n", (unsigned int) event);

  uid = event_get_uid (event);
  if (uid != NULL)
  {
    uid_str = uid2str (uid);
    g_free (uid);
  }
  tmp_str = event_get_description (event);
  g_print ("|---> UID:         %-20s\n", uid_str ? uid_str : "NULL");
  g_print ("|---> DESCRIPTION: %-20s\n", tmp_str ? tmp_str : "NULL");
  g_free (tmp_str);
  tmp_time = event_get_start (event);
  if (tmp_time)
    g_print ("|---> START:       %s\n", ctime (&tmp_time));
  tmp_time = event_get_end (event);
  if (tmp_time)
    g_print ("|---> END:       %s\n", ctime (&tmp_time));

  g_print ("|---> HAS_ALARM:       %d\n", event_get_has_alarm (event));
  g_print ("|---> ALARM_DELAY:       %d\n", event_get_alarm_delay (event));
  g_print ("|---> REPEAT TYPE:     %d\n", event_get_repeat_type (event));
  if (!event_get_repeat_forever (event))
  {
    tmp_time = event_get_repeat_end (event);
    g_print ("|---> REPEAT END:     %s\n", ctime (&tmp_time));

  }

  g_free (uid_str);

  return;
}

/*
vi:ts=2:nowrap:ai:expandtab
*/

