/*  libcalenabler - Library of calendaring and scheduling enabler
 *
 *  Authors: YANG Lin <lin.yang@orange-ftgroup.com> 
 *  
 *  This software and associated documentation files (the "Software") 
 *  are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *  All Rights Reserved. 
 *
 *  A copyright license is hereby granted for redistribution and use of 
 *  the Software in source and binary forms, with or without modification, 
 *  provided that the following conditions are met: 
 *  - Redistributions of source code must retain the above copyright notice, 
 *  this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 *   notice, this copyright license and the following disclaimer in the 
 *  documentation and/or other materials provided with the distribution. 
 *  - Neither the name of LiPS  nor the names of its Members may be used 
 *  to endorse or promote products derived from the Software without 
 *  specific prior written permission. 
 *
 *  A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *  to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *  distribute any implementation compliant with the any specification adopted 
 *  by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *  and non-discriminatory terms and conditions and under reciprocity, as 
 *  regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *  "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *  AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *  ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *  POSSIBILITY OF SUCH DAMAGE. 
 */


#include <glib.h>

#include <calevent.h>

struct _cal_event_iter_t
{
  guint index;
  GPtrArray *array;
};

cal_event_iter_t *
event_iterator_new (void)
{
  cal_event_iter_t *iter = NULL;

  iter = g_new0 (cal_event_iter_t, 1);
  g_assert (iter);

  iter->array = g_ptr_array_new ();
  g_assert (iter->array);
  iter->index = -1;

  return iter;
}

void
event_iterator_free (cal_event_iter_t *iter)
{
  cal_event_t *event;
  gint i, count;

  count = iter->array->len;
  for (i = 0; i < count; i++)
  {
    event = g_ptr_array_remove_index_fast (iter->array, 0);
    event_free (event);
  }

  g_ptr_array_free (iter->array, TRUE);

  g_free (iter);

  return;
}

cal_int_t
event_iterator_size (cal_event_iter_t *iter)
{
  return iter->array->len;
}

cal_boolean_t
event_iterator_at_first (const cal_event_iter_t *iter)
{
  if (iter->index != 0)
  {
    return FALSE;
  }

  return TRUE;
}

cal_boolean_t
event_iterator_at_last (const cal_event_iter_t *iter)
{
  if (iter->index != iter->array->len)
  {
    return FALSE;
  }

  return TRUE;
}

void
event_iterator_to_first (cal_event_iter_t *iter)
{

  if (iter->array->len != 0)
    iter->index = 0;

  return;
}

void
event_iterator_to_last (cal_event_iter_t *iter)
{

  g_assert (iter->array->len != 0);

  iter->index = iter->array->len - 1;

  return;
}

cal_event_t *
event_iterator_current (const cal_event_iter_t *iter)
{
  g_assert (iter->array->len != 0);

  if (iter->index < 0 || iter->index == iter->array->len)
  {
    return NULL;
  }

  return (cal_event_t *) g_ptr_array_index (iter->array, iter->index);
}

void
event_iterator_next (cal_event_iter_t *iter)
{
  g_assert (iter->array->len != 0);

  if (iter->index < iter->array->len)
  {
    iter->index++;
  }

  return;
}

/* *
 * Private functions
 */

void
_event_iterator_insert (cal_event_iter_t *iter, cal_event_t *event)
{

  if (iter->index == -1)
  {
    iter->index = 0;
  }

  g_ptr_array_add (iter->array, (gpointer) event);

  return;
}

/*
vi:ts=2:nowrap:ai:expandtab
*/

